#!/usr/bin/env python

"""Flasker command line tool."""

from argparse import ArgumentParser, REMAINDER
from code import interact
from distutils.dir_util import copy_tree
from functools import wraps
from os import listdir, mkdir
from os.path import abspath, dirname, exists, join, splitext
from re import findall
from shutil import copy
from sys import path

from flasker import current_project
from flasker.project import Project, ProjectImportError

def project_context(app=False, celery=False):
  """Create the project context.
  
  If no configuration file was entered in the main parser through the -c
  option, this function will look in the current directory for possible
  matches. If a single file is found it will use it.

  Some (most) subparser handlers require the project to be created before
  returning, this decorator handles this.

  """
  def _wrapper(handler):
    @wraps(handler)
    def wrapper(*args, **kwargs):
      parsed_args = args[0]
      try:
        conf_file = parsed_args.conf
        if not conf_file:
          conf_files = [fn for fn in listdir('.') if splitext(fn)[1] == '.cfg']
          if len(conf_files) == 0:
            print (
              'No configuration file found in current directory. '
              'Please enter a path to one with the -c option.'
            )
            return
          elif len(conf_files) > 1:
            print (
              'Several configuration files found in current directory: %s. '
              'Please disambiguate with the -c option.'
            ) % ', '.join(conf_files)
            return
          else:
            conf_file = conf_files[0]
        path.append(abspath(dirname(conf_file))) # for reloader to work
        pj = Project(conf_file)
      except ProjectImportError as e:
        print e
        return
      else:
        pj.make(app, celery)
        handler(*args, **kwargs)
    return wrapper
  return _wrapper

# Parsers

# Main parser

parser = ArgumentParser('flasker')

parser.add_argument('-c', '--conf',
  dest='conf',
  default='',
  help='path to configuration file'
)
subparsers = parser.add_subparsers(
  title='available commands',
  dest='command',
)

# New

new_parser = subparsers.add_parser('new', help='start new project')

new_parser.add_argument('-a', '--app',
  action='store_false',
  help='don\'t include basic bootstrap app template'
)
new_parser.add_argument('-n', '--name',
  default='default',
  help='name of the new config file [%(default)s]'
)
new_parser.add_argument('config',
  choices=[
    splitext(name)[0]
    for name in listdir(join(dirname(__file__), 'data', 'configs'))
  ],
  help='the type of config to create'
)

def new_handler(parsed_args):
  src = dirname(__file__)
  conf_name = '%s.cfg' % parsed_args.name
  if exists(conf_name):
    print (
      'There already exists a configuration file with this name. '
      'Please enter a different name with the -n option.'
    )
  else:
    copy(join(src, 'data', 'configs', '%s.cfg' % parsed_args.config), conf_name)
    print 'Project configuration file created!'
  if parsed_args.app:
    if exists('app'):
      print (
        'There already seems to be an app folder here. '
        'App creation skipped.'
      )
    else:
      mkdir('app')
      copy_tree(join(src, 'data', 'app'), 'app')
      print 'Bootstrap app folder created!'

new_parser.set_defaults(handler=new_handler)

# Server

server_parser = subparsers.add_parser('server', help='start server')

server_parser.add_argument('-r', '--restrict',
  action='store_true',
  help='disallow remote server connections'
)
server_parser.add_argument('-p', '--port',
  default=5000,
  type=int,
  help='listen on port [%(default)s]'
)
server_parser.add_argument('-d', '--debug',
  action='store_true',
  help='run in debug mode (autoreload and debugging)'
)

@project_context(app=True)
def server_handler(parsed_args):
  """Start a Werkzeug server for the Flask application."""
  pj = current_project
  host = '127.0.0.1' if parsed_args.restrict else '0.0.0.0'
  pj.app.run(host=host, port=parsed_args.port, debug=parsed_args.debug)

server_parser.set_defaults(handler=server_handler)

# Shell

shell_parser = subparsers.add_parser('shell', help='start shell')

@project_context()
def shell_handler(parsed_args):
  """Start a shell in the context of the project."""
  pj = current_project
  context = {
    'pj': pj,
    'app': pj.app,
    'cel': pj.celery
  }
  try:
    import IPython
  except ImportError:
    interact(local=context)
  else:
    try:
      sh = IPython.Shell.IPShellEmbed()
    except AttributeError:
      sh = IPython.frontend.terminal.embed.InteractiveShellEmbed()
    sh(global_ns=dict(), local_ns=context)

shell_parser.set_defaults(handler=shell_handler)

# Worker

worker_parser = subparsers.add_parser('worker', help='start worker')

worker_parser.add_argument('-o', '--only-direct',
  action='store_true',
  help='only listen to direct queue'
)
worker_parser.add_argument('-v', '--verbose-help',
  action='store_true',
  help='show full help from celery worker'
)
worker_parser.add_argument('-r', '--raw',
  nargs=REMAINDER,
  help='raw options to pass through',
)

@project_context(celery=True)
def worker_handler(parsed_args):
  """Starts a celery worker.

  If no hostname is provided, one will be generated automatically using the
  project domain and subdomain and current worker count. For example, the first
  two workers started for project ``my_project`` and configuration ``default``
  will have respective hostnames:

  * w1.default.my_project
  * w2.default.my_project

  The ``-o`` flag can be used to have the worker only listen to its direct
  queue (this option requires the CELERY_WORKER_DIRECT to be set to ``True``).

  """
  pj = current_project
  pj_worker_names = [d.keys()[0] for d in pj.celery.control.ping()]
  worker_pattern = r'w(\d+)\.%s.%s' % (pj.subdomain, pj.domain)
  worker_numbers = [
    findall(worker_pattern, worker_name) or ['0']
    for worker_name in pj_worker_names
  ]
  wkn = min(
    set(range(1, len(worker_numbers) + 2)) -
    set([int(n[0]) for n in worker_numbers] or [len(worker_numbers) + 2]) 
  )
  if parsed_args.verbose_help:
    pj.celery.worker_main(['worker', '-h'])
  else:
    hostname = 'w%s.%s.%s' % (wkn, pj.subdomain, pj.domain)
    options = ['worker', '--hostname=%s' % hostname]
    if parsed_args.only_direct:
      options.append('--queues=%s.dq' % hostname)
    if parsed_args.raw:
      options.extend(parsed_args.raw)
    pj.celery.worker_main(options)

worker_parser.set_defaults(handler=worker_handler)

# Flower

flower_parser = subparsers.add_parser('flower', help='start flower')

flower_parser.add_argument('-p', '--port',
  default=5555,
  type=int,
  help='listen on port [%(default)s]'
)
flower_parser.add_argument('-v', '--verbose-help',
  action='store_true',
  help='show full help from celery flower'
)
flower_parser.add_argument('-r', '--raw',
  help='raw options to pass through',
  nargs=REMAINDER
)

@project_context()
def flower_handler(parsed_args):
  """Start flower worker manager."""
  pj = current_project
  if parsed_args.verbose_help:
    pj.celery.start(['celery', 'flower', '--help'])
  else:
    options = ['celery', 'flower', '--port=%s' % parsed_args.port]
    if parsed_args.raw:
      options.extend(parsed_args.raw)
    pj.celery.start(options)

flower_parser.set_defaults(handler=flower_handler)

# END of parsers

def main():
  parsed_args = parser.parse_args()
  parsed_args.handler(parsed_args)

if __name__ == '__main__':
  main()



#!/usr/bin/env python

"""Creating the Celery application."""

from __future__ import absolute_import

from celery import Celery
from celery.signals import worker_process_init
from celery.task import periodic_task

from ..project import current_project

pj = current_project

celery = Celery()
celery.conf.update(pj.config['CELERY'])
celery.periodic_task = periodic_task

@worker_process_init.connect
def create_worker_connection(*args, **kwargs):
  """Initialize database connection.

  This has to be done after the worker processes have been started otherwise
  the connection will fail.

  """
  pj.setup_database_connection(celery=True)

pj.celery = celery



#!/usr/bin/env python

"""API Manager."""

from flask import Blueprint, jsonify, request
from os.path import abspath, dirname, join
from sqlalchemy import Column
from sqlalchemy.ext.declarative import declarative_base, declared_attr 
from sqlalchemy.orm import class_mapper, mapperlib, Query
from sqlalchemy.orm.collections import InstrumentedList
from sqlalchemy.orm.properties import RelationshipProperty
from time import time
from werkzeug.exceptions import HTTPException

from ..project import current_project as pj
from ..util import Cacheable, JSONEncodedDict, _jsonify, Loggable, uncamelcase

class _BaseQuery(Query):

  """Base query class.

  From Flask-SQLAlchemy.

  """

  def get_or_404(self, model_id):
    """Like get but aborts with 404 if not found."""
    rv = self.get(model_id)
    if rv is None:
      abort(404)
    return rv

  def first_or_404(self):
    """Like first but aborts with 404 if not found."""
    rv = self.first()
    if rv is None:
      abort(404)
    return rv

class _QueryProperty(object):

  def __init__(self, project):
    self.project = project

  def __get__(self, obj, cls):
    try:
      mapper = class_mapper(cls)
      if mapper:
        return _BaseQuery(mapper, session=self.project.session())
    except UnmappedClassError:
      return None

class ExpandedBase(Cacheable, Loggable):

  """Adding a few features to the declarative base.

  Currently:

  * Automatic table naming
  * Caching
  * Jsonifying
  * Logging

  """

  _cache = Column(JSONEncodedDict)
  _json_depth = -1

  json_exclude = None
  json_include = None
  query = None

  def __init__(self, **kwargs):
    for k, v in kwargs.items():
      setattr(self, k, v)

  def __repr__(self):
    primary_keys = ', '.join(
      '%s=%r' % (k, getattr(self, k))
      for k in self.__class__.get_primary_key_names()
    )
    return '<%s (%s)>' % (self.__class__.__name__, primary_keys)

  @declared_attr
  def __tablename__(cls):
    """Automatically create the table name.

    Override this to choose your own tablename (e.g. for single table
    inheritance).

    """
    return '%ss' % uncamelcase(cls.__name__)

  @declared_attr
  def _json_attributes(cls):
    """Create the dictionary of attributes that will be JSONified.

    This is only run once, on class initialization, which makes jsonify calls
    much faster.

    By default, includes all public (don't start with _):

    * properties
    * columns that aren't foreignkeys.
    * joined relationships (where lazy is False)

    """
    rv = set(
        varname for varname in dir(cls)
        if not varname.startswith('_')  # don't show private properties
        if (
          isinstance(getattr(cls, varname), property) 
        ) or (
          isinstance(getattr(cls, varname), Column) and
          not getattr(cls, varname).foreign_keys
        ) or (
          isinstance(getattr(cls, varname), RelationshipProperty) and
          not getattr(cls, varname).lazy == 'dynamic'
        )
      )
    if cls.json_include:
      rv = rv | set(cls.json_include)
    if cls.json_exclude:
      rv = rv - set(cls.json_exclude)
    return list(rv)

  def jsonify(self, depth=0):
    """Special implementation of jsonify for Model objects.
    
    Overrides the basic jsonify method to specialize it for models.

    This function minimizes the number of lookups it does (no dynamic
    type checking on the properties for example) to maximize speed.

    :param depth:
    :type depth: int
    :rtype: dict

    """
    if depth <= self._json_depth:
      # this instance has already been jsonified at a greater or
      # equal depth, so we simply return its key
      return self.get_primary_keys()
    rv = {}
    self._json_depth = depth
    for varname in self._json_attributes:
      try:
        rv[varname] = _jsonify(getattr(self, varname), depth)
      except ValueError as e:
        rv[varname] = e.message
    return rv

  def get_primary_keys(self):
    return dict(
      (k, getattr(self, k))
      for k in self.__class__.get_primary_key_names()
    )

  @classmethod
  def find_or_create(cls, **kwargs):
    instance = self.filter_by(**kwargs).first()
    if instance:
      return instance, False
    instance = cls(**kwargs)
    session = cls.query.db.session
    session.add(instance)
    session.flush()
    return instance, True

  @classmethod
  def get_columns(cls, show_private=False):
    columns = class_mapper(cls).columns
    if not show_private:
      columns = [c for c in columns if not c.key.startswith('_')]
    return columns

  @classmethod
  def get_relationships(cls):
    return class_mapper(cls).relationships

  @classmethod
  def get_related_models(cls):
    return [(k, v.mapper.class_) for k, v in cls.get_relationships().items()]

  @classmethod
  def get_primary_key_names(cls):
    return [key.name for key in class_mapper(cls).primary_key]

Model = declarative_base(cls=ExpandedBase)

class APIError(HTTPException):

  """Thrown when an API call is invalid."""

  def __init__(self, code, content):
    self.code = code
    self.content = content
    super(APIError, self).__init__(content)

  def __repr__(self):
    return '<APIError %r: %r>' % (self.message, self.content)

class APIManager(object):

  """Main API manager.

  Handles the creation and registration of all API views. Available options:

  * URL_PREFIX
  * ADD_ALL_MODELS
  * DEFAULT_METHODS
  * DEFAULT_ALLOW_PUT_MANY
  * DEFAULT_RELATIONSHIPS
  * DEFAULT_COLLECTION_DEPTH
  * DEFAULT_MODEL_DEPTH
  * DEFAULT_LIMIT
  * MAX_LIMIT

  Models can be added in two ways. Either individually::
    
    manager.add_model(Model)

  or globally::

    manager.add_all_models()

  Both functions accept the same additional options (cf. their respective doc).

  It also exposes the `authorize` and `validate` decorators.

  Once all the models have been added along with (optionally) the authorize and
  validate functions, the manager should be registered with the project::

    current_project.register_manager(manager)

  """

  _authorize = None
  _validate = None

  config = {
    'URL_PREFIX': '/api',
    'ADD_ALL_MODELS': False,
    'METHODS': frozenset(['GET', 'POST', 'PUT', 'DELETE']),
    'ALLOW_PUT_MANY': False,
    'RELATIONSHIPS': True,
    'DEFAULT_COLLECTION_DEPTH': 0,
    'DEFAULT_MODEL_DEPTH': 1,
    'DEFAULT_LIMIT': 20,
    'MAX_LIMIT': None,
  }

  def __init__(self, **kwargs):
    for k, v in kwargs.items():
      self.config[k.upper()] = v
    self.Models = {}
    APIView._manager = self

  def add_model(self, Model, relationships=None, allow_put_many=None,
                methods=None):
    """Flag a Model to be added.
    
    Will override any options previously set for that Model.

    If any of the options is ``None``, the default value will be used.

    Relationships can either be ``True`` or a list of relationship keys. In the
    first case, all one to many relationships will have a hook created,
    otherwise only those mentioned in the list.

    It might seem strange that this method doesn't have a columns filter, this
    is for speed and consistency. Columns are defined at the Model level (via
    the json_include and json_exclude attributes), this way:
    
    * jsonify calls do not have to dynamically check which columns to include
    * all endpoints leading to the same Model yield the same columns

    """
    self.Models[Model.__name__] = (
      Model, methods, relationships, allow_put_many
    )

  def authorize(self, func):
    """Decorator to set the authorizer function.

    The authorizer is passed three arguments:
    
    * the model class
    * the relationship (or ``None``)
    * the request method 

    The third argument is for convenience (the full request object can be 
    accessed as usual).

    If the function returns a truthful value, the request will proceed.
    Otherwise a 403 exception is raised.
    
    """
    self._authorize = func

  def validate(self, func):
    """Decorator to set the validate function, called on POST and PUT.

    The validator will be passed two arguments:

    * the model class
    * the request json
    * the request method

    If the function returns a truthful value, the request will proceed.
    Otherwise a 400 exception is raised.

    """
    self._validate = func

  def _create_model_views(self, Model, relationships, allow_put_many, methods):
    """Creates the views associated with the model.

    Sane defaults are chosen:

    * PUT and DELETE requests are only allowed for endpoints that correspond
      to a single model
    * POST requests are only allowed for endpoints that correspond to
      collections
    * endpoints corresponding to relationships only allow the GET method
      (this choice was made to avoid duplicating features accross endpoints)

    """
    views = []
    methods = set(methods)
    model_methods = methods & set(['GET', 'PUT', 'DELETE'])
    if model_methods:
      views.append(ModelView(Model=Model, methods=model_methods))
    collection_methods = methods & set(['GET', 'POST'])
    if collection_methods:
      views.append(CollectionView(Model=Model, methods=collection_methods))
    if allow_put_many:
      col_methods = set(['GET', 'PUT']) & methods
    else:
      col_methods = set(['GET']) & methods
    rel_methods = set(['GET']) & methods
    if rel_methods:
      for rel in Model.get_relationships().values():
        if rel.key in relationships and rel.uselist:
          views.extend([
            ModelView(Model=Model, relationship=rel, methods=rel_methods),
            CollectionView(Model=Model, relationship=rel, methods=col_methods)
          ])
    for view in views:
      self.blueprint.add_url_rule(view.url, view.endpoint, view)

  def _before_register(self, project):
    self.blueprint = Blueprint(
      'api',
      project.config['PROJECT']['APP_FOLDER'] + '.api',
      template_folder=abspath(join(dirname(__file__), 'templates', 'api')),
      url_prefix=self.config['URL_PREFIX']
    )
    if self.config['ADD_ALL_MODELS']:
      Models = [k.class_ for k in mapperlib._mapper_registry]
      for Model in Models:
        if not Model.__name__ in self.Models:
          self.add_model(Model)
    for Model, methods, rels, batch in self.Models.values():
      if methods is None: methods = self.config['METHODS']
      if batch is None: batch = self.config['ALLOW_PUT_MANY']
      if rels is None: rels = self.config['RELATIONSHIPS']
      if rels is True:
        rels = set(Model.get_relationships().keys())
      elif rels is False:
        rels = set()
      self.Models[Model.__name__] = (Model, methods, rels, batch)
      self._create_model_views(Model, rels, batch, methods)
    index_view = IndexView()
    self.blueprint.add_url_rule(
      index_view.url, index_view.endpoint, index_view
    )

  def _after_register(self, project):
    Model.metadata.create_all(project._engine, checkfirst=True)
    Model.query = _QueryProperty(project)

class APIView(object):

  """Base API view.

  Note that the methods class attribute seems to be passed to the add_url_rule
  function somehow (!).
  
  """

  __all__ = []

  _manager = None

  Model = None
  relationship = None
  methods = set()
  params = set()

  def __init__(self, **kwargs):
    for k, v in kwargs.items():
      setattr(self, k, v)
    self.__all__.append(self)

  def __call__(self, **kwargs):
    """Redirects to corresponding request handler and catch APIErrors."""
    try:
      if not self.is_authorized():
        raise APIError(403, 'Not authorized')
      if request.method in self.methods:
        params = self._parse_params()
        return getattr(self, request.method.lower())(params, **kwargs)
      else:
        raise APIError(405, 'Method Not Allowed')
    except APIError as e:
      return jsonify({
        'status': e.message,
        'request': {
          'base_url': request.base_url,
          'method': request.method,
          'values': request.values
        },
        'content': e.content
      }), e.code

  @property
  def available_columns(self):
    return self.Model._json_attributes

  @property
  def available_relationships(self):
    return [
      r for r in self.Model.get_relationships()
      if r.uselist
      if r.key in self._manager.Models[self.Model.__name__][2]
    ]

  @property
  def authorized_methods(self):
    return set(
      method for method in self.methods 
      if self._manager._authorize(self.Model, self.relationship, method)
    )

  @property
  def defaults(self):
    return self._manager.config

  def is_validated(self, json):
    """Validate changes."""
    if not self._manager._validate:
      return True
    return self._mananager._validate(self.Model, json, request.method)

  def is_authorized(self):
    if not self._manager._authorize:
      return True
    return self._manager._authorize(
      self.Model, self.relationship, request.method
    )

  def _parse_params(self):
    return request.args

class IndexView(APIView):

  """API 'splash' page with a few helpful keys."""

  methods = set(['GET'])

  @property
  def endpoint(self):
    return 'index'
 
  @property
  def url(self):
    return '/'

  def get(self, params, **kwargs):
    return jsonify({
      'status': '200 Welcome',
      'available_endpoints': [
        '%s (%s)' % (v.url, ', '.join(v.authorized_methods))
        for v in self.__all__
        if v.authorized_methods
      ],
      'available_models': dict(
        (
          v.Model.__name__,
          {
            'available_columns': v.available_columns,
            'available_relationships': [
              r.key for r in v.available_relationships
            ]
          }
        ) for v in self.__all__
        if v.Model
        if not v.relationship
      )
    })

class CollectionView(APIView):

  """View for collection endpoints."""

  params = set(['depth', 'limit', 'offset', 'loaded', 'sort'])

  @property
  def endpoint(self):
    if not self.relationship:
      return '%s_collection_view' % self.Model.__tablename__
    else:
      return '%s_%s_collection_view' % (
        self.Model.__tablename__, self.relationship.key
      )

  @property
  def url(self):
    url = '/%s' % self.Model.__tablename__
    if self.relationship:
      url += ''.join('/<%s>' % n for n in self.Model.get_primary_key_names())
      url += '/%s' % self.relationship.key
    url += '/'
    return url

  def get(self, params, **kwargs):
    if not kwargs:
      query = self.Model.query
    else:
      model = self.Model.query.get(kwargs.values())
      if not model:
        raise APIError(404, 'No resource found for this ID')
      query = getattr(model, self.relationship.key)
    if isinstance(query, InstrumentedList):
      results = self._process_list(query, params)
    else:
      results = self._process_query(query, params)
    return jsonify({
      'status': '200 Success',
      'processing_time': results['processing_times'],
      'matches': {
        'total': results['total_matches'],
        'returned': len(results['content'])
      },
      'request': {
        'base_url': request.base_url,
        'method': request.method,
        'values': request.values
      },
      'content': results['content']
    }), 200

  def post(self, params, **kwargs):
    if self.is_validated(request.json):
      model = Model(**request.json)
      pj.session.add(model)
      pj.session.commit() # generate an ID
      return jsonify(model.jsonify(depth=params['depth']))
    else:
      raise APIError(400, 'Failed validation')

  def put(self, params, **kwargs):
    parent = self.Model.query.get(kwargs.values())
    if parent:
      models = getattr(model, self.relationship.key)
      if self.is_validated(request.json):
        for model in models:
          for k, v in request.json.items():
            setattr(model, k, v)
        return jsonify(parent.jsonify(depth=params['depth']))
      else:
        raise APIError(400, 'Failed validation')
    else:
      raise APIError(404, 'No resource found for this ID')

  def _process_list(self, lst, params):
    return {
      'processing_times': [],
      'total_matches': len(lst),
      'content': [e.jsonify(self.defaults['DEFAULT_COLLECTION_DEPTH']) for e in lst]
    }

  def _process_query(self, query, params):
    Model = query.column_descriptions[0]['type']
    column_names = [c.key for c in Model.get_columns()]
    timer = time()
    processing_times = []
    filters = {}
    for k, v in request.args.items():
      if k in column_names:
        filters[k] = v
      elif not k in self.params:
        raise APIError(400, 'Bad Request')
    try:
      offset = max(0, int(request.args.get('offset', 0)))
      limit = max(0, int(
        request.args.get('limit', self.defaults['DEFAULT_LIMIT'])
      ))
      if self.defaults['MAX_LIMIT']:
        limit = min(limit, self.defaults['MAX_LIMIT'])
      depth = max(0, int(
        request.args.get('depth', self.defaults['DEFAULT_COLLECTION_DEPTH'])
      ))
      loaded = [int(e) for e in request.args.get('loaded', '').split(',') if e]
      sort = request.args.get('sort', '')
    except ValueError as e:
      raise APIError(400, 'Invalid parameters')
    processing_times.append(('request', time() - timer))
    timer = time()
    for k, v in filters.items():
      query = query.filter(getattr(Model, k) == v)
    total_matches = 10 # query.count()
    processing_times.append(('query', time() - timer))
    timer = time()
    if loaded:
      query = query.filter(~Model.id.in_(loaded))
    if sort:
      attr = sort.strip('-')
      if not attr in column_names:
        raise APIError(400, 'Invalid sort parameter')
      if sort[0] == '-':
        query = query.order_by(-getattr(Model, attr))
      else:
        query = query.order_by(getattr(Model, attr))
    if limit:
      query = query.limit(limit)
    return {
      'processing_times': processing_times,
      'total_matches': total_matches,
      'content': [
        e.jsonify(depth=depth)
        for e in query.offset(offset)
      ]
    }

class ModelView(APIView):

  """View for individual model endpoints."""

  params = set(['depth'])

  @property
  def endpoint(self):
    if not self.relationship:
      return '%s_model_view' % self.Model.__tablename__
    else:
      return '%s_%s_model_view' % (
        self.Model.__tablename__, self.relationship.key
      )

  @property
  def url(self):
    url = '/%s' % self.Model.__tablename__
    url += ''.join('/<%s>' % n for n in self.Model.get_primary_key_names())
    if self.relationship:
      url += '/%s/<position>' % self.relationship.key
    return url

  def get(self, params, **kwargs):
    position = kwargs.pop('position', None)
    depth = int(request.args.get('depth', self.defaults['DEFAULT_MODEL_DEPTH']))
    model = self.Model.query.get(kwargs.values())
    if position is not None:
      try:
        pos = int(position) - 1
      except ValueError:
        raise APIError(400, 'Invalid position index')
      else:
        if pos >= 0:
          query_or_list = getattr(model, self.relationship.key)
          if isinstance(query_or_list, InstrumentedList):
            try:
              model = query_or_list[pos]
            except IndexError:
              model = None
          else:
            model = query_or_list.offset(pos).first()
        else:
          raise APIError(400, 'Invalid position index')
    if not model:
      raise APIError(404, 'No resource at this position')
    return jsonify(model.jsonify(depth=depth))

  def put(self, params, **kwargs):
    model = self.Model.query.get(kwargs.values())
    if model:
      if self.is_validated(request.json):
        for k, v in request.json.items():
          setattr(model, k, v)
        return jsonify(model.jsonify(depth=params['depth']))
      else:
        raise APIError(400, 'Failed validation')
    else:
      raise APIError(404, 'No resource found for this ID')

  def delete(self, params, **kwargs):
    model = self.Model.query.get(kwargs.values())
    if model:
      pj.session.delete(model)
      return jsonify({'status': '200 Success', 'content': 'Resource deleted'})
    else:
      raise APIError(404, 'No resource found for this ID')




#!/usr/bin/env python

"""General helpers."""

from collections import defaultdict
from csv import DictReader
from datetime import datetime
from json import dumps, loads
from functools import partial, wraps
from re import sub
from sqlalchemy.ext.mutable import Mutable
from sqlalchemy.types import TypeDecorator, UnicodeText
from time import time


# Utility functions

def prod(iterable, key=None):
  """Helper function for cumulative products.

  :param key: function called on each element of the iterable, if none then
    identity is assumed
  :type key: callable
  :param restrict: if provided, elements where restrict(element) is True
    will not be included
  :type restrict: callable

  """
  rv = 1
  for index, elem in enumerate(iterable):
    if key is None:
      rv *= elem
    else:
      rv *= key(elem)
  return rv

def uncamelcase(name):
  """Transform CamelCase to underscore_case."""
  s1 = sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
  return sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

def convert(value, return_type=None):
  """Converts a string to a boolean, int or float."""
  value = value.strip()
  if return_type:
    if return_type == 'int':
      return int(value)
    elif return_type == 'float':
      return float(value)
    elif return_type == 'bool':
      if value.lower() == 'true' or value == '1':
        return True
      elif not value or value.lower() == 'false' or value == '0':
        return False
      else:
        raise ValueError('Can\'t convert %s to boolean.' % value)
    elif return_type == 'unicode':
      return unicode(value, encoding='utf-8', errors='replace')
    elif return_type == 'str':
      return value
    # if we get here, something has gone wrong
    raise ValueError('Invalid conversion type: %s' % return_type)
  else:
    try:
      return int(value)
    except ValueError:
      try:
        return float(value)
      except ValueError:
        if value.lower() == 'true':
          return True
        elif value.lower() == 'false':
          return False
        else:
          return value


# Utility Classes

class Dict(dict):

  """Dictionary class with a few helper methods.

  The goal of this class is to make multilevel dictionary actions
  simple.

  Usage::

    d = Dict(d)
    d.flattened()

  :param cname: key used when unflattening a dictionary and a key with a
    value also becomes a branch
  :type cname: string
  :param sep: the separator used to separate hierarchy levels
  :type sep: string

  """

  cname = 'all'
  sep = '_'

  def depth(self):
    """Depth of a dictionary."""
    values = [
        Dict(value)
        for value in self.itervalues()
        if isinstance(value, dict)
    ]
    return max(value.depth() for value in values) + 1 if values else 1

  def width(self):
    """Width of a dictionary."""
    values = [
        Dict(value)
        for value in self.itervalues()
        if isinstance(value, dict)
    ]
    return sum(value.width() for value in values) + len(self) - len(values)

  def table(self, mode='horizontal', left_columns=None):
    """For HTML headers mostly."""
    items = []
    unflattened = Dict(self.unflattened())
    depth = unflattened.depth()
    width = unflattened.width()
    if mode == 'horizontal':
      levels = defaultdict(list)
      for key in sorted(self.flattened().iterkeys()):
        parts = key.split(self.sep)
        for index, part in enumerate(parts[:-1]):
          levels[index].append([part, 1, 1, self.sep.join(parts[:(index + 1)])])
        levels[len(parts) - 1].append([parts[-1], depth - len(parts) + 1, 1, key])
      for index, level in levels.items():
        if index == 0 and left_columns:
          row = [[column, depth, 1, column] for column in left_columns]
        else:
          row = []
        current_label = None
        for label, height, width, full_label in level:
          if label == current_label:
            row[-1][2] += 1
          else:
            current_label = label
            row.append([label, height, width, full_label])
        items.append(row)
    elif mode == 'vertical':
      indices = {}
      for i, key in enumerate(sorted(self.flattened().iterkeys())):
        if i == 0 and left_columns:
          row = [[column, width, 1, column] for column in left_columns]
        else:
          row = []
        parts = key.split(self.sep)
        k = 0
        for j, part in enumerate(parts[:-1]):
          full_label = self.sep.join(parts[:(j + 1)])
          if not full_label in indices:
            indices[full_label] = (i, k)
            row.append([part, 1, 1, full_label])
            k += 1
          else:
            a, b = indices[full_label]
            items[a][b][1] += 1
        indices[key] = (i, k)
        row.append([parts[-1], 1, depth - len(parts) + 1, key])
        items.append(row)
    return items

  def flattened(self):
    """Flattened representation of the dictionary."""
    return self.__class__.flatten(self)

  def unflattened(self):
    """Unflattened representation of the dictionary."""
    return self.__class__.unflatten(self)

  @classmethod
  def flatten(cls, dic, sep=None, prefix=''):
    """Flatten. Classmethod for convenience."""
    sep = sep if sep else cls.sep
    items = []
    for key, value in dic.iteritems():
      k = prefix + sep + key if prefix else key
      if isinstance(value, dict) and value:
        items.extend(cls.flatten(value, sep, k).items())
      else:
        items.append((k, value))
    return dict(items)

  @classmethod
  def unflatten(cls, dic, sep=None, cname=None):
    """Unflatten. Classmethod for convenience"""
    sep = sep if sep else cls.sep
    cname = cname if cname else cls.cname
    result = {}
    keys = []
    for key in dic.iterkeys():
      keys.append(key.split(sep))
    keys.sort(key=len, reverse=True)
    for key in keys:
      d = result
      for part in key[:-1]:
        if part not in d:
          d[part] = {}
        d = d[part]
      if key[-1] in d:
        d[key[-1]][cname] = dic[sep.join(key)]
      else:
        d[key[-1]] = dic[sep.join(key)]
    return result

class SmartDictReader(DictReader):

  """Helper for importing .csv files.

  :param csvfile: open file instance
  :type csvfile: ``file``
  :param fields: list of fieldnames or list of tuples (fieldname, fieldtype)
  :rtype: iterable

  Interesting values for kwargs can be:
  * delimiter = '\t'
  * quotechar = '\x07'

  """

  def __init__(self, csvfile, fields=None, silent=False, **kwargs):
    self.csvfile = csvfile
    self.rows_imported = 0
    self.errors = []
    self.silent = silent
    if fields:
      if isinstance(fields[0], (list, tuple)):
        kwargs['fieldnames'] = [field[0] for field in fields]
        self.field_types = dict(fields)
      else:
        kwargs['fieldnames'] = fields
        self.field_types = dict.fromkeys(fields, None)
      DictReader.__init__(self, csvfile, **kwargs)
    else:
      DictReader.__init__(self, csvfile, **kwargs)
      self.field_types = dict.fromkeys(self.fieldnames, None)

  def next(self):
    row = DictReader.next(self)
    try:
      processed_row = dict(
          (key, convert(value, self.field_types[key]))
          for key, value in row.iteritems()
      )
    except ValueError as e:
      self.errors.append((e, row))
      if not self.silent:
        raise e
    else:
      self.rows_imported += 1
      return processed_row


# Caching

def cached_property(func):
  return _CachedProperty(func)

class Cacheable(object):

  """Adds a few cache maintenance utilities."""

  def _get_cached_properties(self):
    return [
        varname
        for varname in dir(self.__class__)
        if isinstance(getattr(self.__class__, varname), _CachedProperty)
    ]

  def refresh_cached_property(self, name, expiration=0):
    setattr(self, name, _CacheRefresh(expiration))

  def refresh_all_cached_properties(self, expiration=0):
    for varname in self._get_cached_properties():
      setattr(self, varname, _CacheRefresh(expiration))

class _CacheRefresh(object):

  def __init__(self, expiration):
    self.expiration = expiration

class _CachedProperty(property):

  """Instance of a cached property for a model.

  Based on the emulation of PyProperty_Type() in Objects/descrobject.c from 
  http://infinitesque.net/articles/2005/enhancing%20Python's%20property.xhtml

  """

  def __init__(self, func):
    self.func = func
    self.__doc__ = func.__doc__

  def __get__(self, obj, objtype=None):
    """Gets the value from cache (creating and refreshing as necessary)."""
    if obj is None:
      return self
    else:
      if not obj._cache or not self.func.__name__ in obj._cache:
        return None
      return obj._cache[self.func.__name__][0]

  def __set__(self, obj, value):
    """Sets the value in the cache.

    If the _cache is a JSONEncodedDict, this will also mark the dictionary
    as changed.
    
    """
    if not hasattr(obj, '_cache') or not obj._cache:
      obj._cache = {}
    if value:
      if isinstance(value, _CacheRefresh):
        t = time() - obj._cache[self.func.__name__][1]
        if t > value.expiration:
          obj._cache[self.func.__name__] = (self.func(obj), time())
      else:
        obj._cache[self.func.__name__] = (value, time())
      try:
        obj._cache.changed()
      except:
        pass

  def __repr__(self):
    return '<CachedProperty %r>' % self.func


# Jsonifification

def _jsonify(value, depth=0):
  if hasattr(value, 'jsonify'):
    return value.jsonify(depth=depth - 1)
  if isinstance(value, dict):
    return dict((k, jsonify(v, depth)) for k, v in value.items())
  if isinstance(value, list):
    return [jsonify(v, depth) for v in value]
  if isinstance(value, (float, int, long, str, unicode, tuple)):
    return value
  if isinstance(value, datetime):
    return str(value)
  if value is None:
    return None
  raise ValueError('not jsonifiable')

class Jsonifiable(object):

  """For easy API calls.

  This is depth first (different from the one in the ExpandedBase class).

  We keep track of what has already been jsonified. There might still be
  some repetition given that we do not control the order of exploration.
  If a key is revisited at a lower depth, it will be reparsed to allow for
  more exploration.

  """

  @property
  def _json_attributes(self):
    """Default implementation of the attributes to jsonify.

    This is relatively slow (because it is evaluated for each jsonify call).
    Consider overwriting it for better performance.
    
    """
    return [
      varname for varname in dir(self)
      if not varname.startswith('_')
      if not hasattr(getattr(self, varname), '__call__')
    ]

  def jsonify(self, depth=0):
    """Returns all keys and properties of an instance in a dictionary.

    Overrides the basic jsonify method to specialize it for models.

    This function minimizes the number of lookups it does (no dynamic
    type checking on the properties for example) to maximize speed.

    :param depth:
    :type depth: int
    :rtype: dict

    """
    rv = {}
    for varname in self._json_attributes:
      try:
        rv[varname] = _jsonify(getattr(self, varname), depth)
      except ValueError as e:
        rv[varname] = e.message
    return rv


# Logging

class Loggable(object):

  """To easily log stuff.

  This implements the main logging methods ('debug', 'info', 'warn', 'error')
  directly on the class instance. For example, this allows something like::

    instance.log('Some message.')

  Not using __getattr__ to preserve exception context. Otherwise the line where
  the inexistent attribute was accessed will be lost.

  """

  def _logger(self, message, loglevel):
    if not hasattr(self, '__logger__'):
      self.__logger__ = getLogger(self.__module__)
    action = getattr(self.__logger__, loglevel)
    return action('%s :: %s' % (self, message))

  def debug(self, message):
    return self._logger(message, 'debug')

  def info(self, message):
    return self._logger(message, 'info')

  def warn(self, message):
    return self._logger(message, 'warn')

  def error(self, message):
    return self._logger(message, 'error')


# SQLAlchemy Mutables

class JSONEncodedDict(TypeDecorator):

  """Represents an immutable structure as a JSON encoded dict.

  This can be used as a Column type during table creation::

    some_column_name = Column(JSONEncodedDict)

  .. note::

    There is a character limit in the UnicodeText field of the database
    so care is needed when storing very large dictionaries.

  """

  impl = UnicodeText

  def process_bind_param(self, value, dialect):
    return dumps(value) if value else None

  def process_result_value(self, value, dialect):
    return loads(value) if value else {}

class _MutableDict(Mutable, dict):

  """Used with JSONEncoded dict to be able to track updates.

  This enables the database to know when it should update the stored string
  representation of the dictionary. This is much more efficient than naive
  automatic updating after each query.

  .. note::

    Only set, del and update actions are tracked. If another method to
    update the dictionary is used, it will not automatically flag the
    dictionary for update (for example if a deeply nested key is updated).
    In such a case, the ``changed`` method needs the be called manually
    after the operation.

  """

  @classmethod
  def coerce(cls, key, value):
    """Convert plain dictionaries to Features."""
    if not isinstance(value, cls):
      if isinstance(value, dict):
        return cls(value)
      return Mutable.coerce(key, value) # this will raise an error
    else:
      return value

  def update(self, *args, **kwargs):
    """Detect dictionary update events and emit change events."""
    dict.update(self, *args, **kwargs)
    self.changed()
    
  def __setitem__(self, key, value):
    """Detect dictionary set events and emit change events."""
    dict.__setitem__(self, key, value)
    self.changed()
    
  def __delitem__(self, key):
    """Detect dictionary del events and emit change events."""
    dict.__delitem__(self, key)
    self.changed()
    
_MutableDict.associate_with(JSONEncodedDict)


# Computations

class RunningStatistic(object):

  """ To compute running statistics efficiently."""

  def __init__(self):
    self.count = 0
    self.mean = float(0)
    self.unweighted_variance = float(0)

  def push(self, n):
    if n == None:
      return
    self.count += 1
    if self.count == 1:
      self.mean = float(n)
      self.unweighted_variance = float(0)
    else:
      mean = self.mean
      s = self.unweighted_variance
      self.mean = mean + (n - mean) / self.count
      self.unweighted_variance = s + (n - self.mean) * (n - mean)

  def variance(self):
      if self.count>1:
        return self.unweighted_variance/(self.count-1)
      return 0

def exponential_smoothing(data, alpha=0.5):
  """Helper function for smoothing data.

  :param data: list of tuples. The smoothing will be done on
    the first item of each tuple.
  :type data: ``list``
  :param alpha: the discount factor
  :type alpha: ``float``
  :rtype: ``list``
  
  """
  sorted_data = sorted(data, key=lambda e: e[0])
  return [(x, sum(_y * alpha ** (x - _x) for (_x, _y) in sorted_data[:i+1])
        / sum(alpha ** (x - _x) for (_x, _y) in sorted_data[:i+1]))
      for i, (x, y) in enumerate(sorted_data)]

def histogram(data, key=lambda a: a, bins=50, restrict=None, categories=None,
              order=0, expand=False):
  """Returns a histogram of counts for the data.

    :param restrict: if provided, only data elements which return `True`
      will be included in the histogram. Default is `None` (all elements
      are included).
    :type restrict: function or None
    :param categories: if provided, elements will be counted in separate
      categories. This changes the format of the output to a dictionary
      with the different categories as keys in each bin.
    :type categories: function or None
    :param bins: either an int (total number of bins, which will be 
      uniformly spread) or a list of increasing bin values. smaller
      values will be in the first bin, larger in the last one.
    :type bins: int or list(int)
    :param order: 0 if data isn't sorted, 1 if sorted in ascending, -1 if
      sorted in descending order.
    :type order: string

  Possible extension: allow categories to return a list of keys, which would
  allow elements to be included in several counts.

  """
  if isinstance(bins, int):
    n_bins = bins
    if not n_bins > 0: raise Exception("Number of bins must be > 0.")
    if order == '1':
      max_value = key(data[-1])
      min_value = key(data[0])
    elif order == '-1':
      max_value = key(data[0])
      min_value = key(data[-1])
    else:
      max_value = max(key(e) for e in data)
      min_value = min(key(e) for e in data)
    if n_bins == 1 or max_value == min_value:
      # If everything is equal, or just one bin, return one bin. Duh.
      return {min_value: len(data)}
    else:
      bin_width = float(max_value - min_value) / n_bins
      bins = [min_value + float(i) * bin_width for i in xrange(n_bins)]
      def find_bin(e):
        # this is faster than default iterating over bins
        index = min(int((key(e) - min_value) / bin_width), n_bins - 1)
        return bins[index]
  else:
    if len(bins) == 1:
      # not very interesting but for compatibility
      return {bins[0]: len(data)}
    def find_bin(a):
      # default bin iterator
      if a < bins[0]:
        return bins[0]
      for bin in reversed(bins):
        if a >= bin:
          return bin
  if categories is None:
    data_histogram = dict.fromkeys(bins, 0)
    for e in data:
      if restrict is None or restrict(e):
        data_histogram[find_bin(key(e))] += 1
    return data_histogram
  else:
    data_histogram = defaultdict(lambda: defaultdict(int))
    for e in data:
      if restrict is None or restrict(e):
        data_histogram[find_bin(key(e))][categories(e)] += 1
    data_histogram = dict((k, dict(v)) for (k, v) in data_histogram.iteritems())
    if expand:
      keys = set(key for v in data_histogram.values() for key in v.keys())
      data_histogram = dict(
          (key, dict((k, v.get(key, 0)) for (k, v) in data_histogram.iteritems()))
          for key in keys
      )
    return data_histogram




#!/usr/bin/env python

"""Creating the Flask application."""

from __future__ import absolute_import

from flask import Flask

from ..project import current_project

pj = current_project
conf = pj.config['PROJECT']

app = Flask(
  conf['APP_FOLDER'],
  static_folder=conf['APP_STATIC_FOLDER'],
  template_folder=conf['APP_TEMPLATE_FOLDER'],
  instance_path=pj.root_dir,
  instance_relative_config=True
)
app.config.update(pj.config['APP'])

@app.context_processor
def inject():
  def static_url(request):
    return conf['STATIC_URL'] or request.url_root + conf['APP_STATIC_FOLDER']
  return {
    'project_name': conf['NAME'],
    'static_url': static_url,
  }

pj.app = app



#!/usr/bin/env python

"""Auth Manager."""

from flask import (Blueprint, current_app, flash, request, redirect,
  render_template, url_for)
from flask.ext.login import (current_user, login_user, logout_user,
  LoginManager, UserMixin)
from json import loads
from os.path import abspath, join, dirname
from urllib import urlencode
from urllib2 import Request, urlopen

from ..util import Loggable

class User(Loggable, UserMixin):

  """Base user class."""

  __all__ =  {}

  def __init__(self, email):
    self.id = email

  def __repr__(self):
    return '<User id=%r>' % self.id

  @property
  def __logger__(self):
    return current_app.logger

  def get_id(self):
    """Necessary for Flask login extension."""
    return self.id

  @classmethod
  def get_from_id(cls, id):
    if id in cls.__all__:
      rv = cls.__all__[id]
    else:
      rv = None
    return rv

class AuthManager(object):

  config = {
    'URL_PREFIX': '/auth',
    'PROTECT_ALL_VIEWS': True,
  }

  def __init__(self, **kwargs):
    for k, v in kwargs.items():
      self.config[k.upper()] = v

  def _create_blueprint(self, project):

    return Blueprint(
      'auth',
      project.config['PROJECT']['APP_FOLDER'] + '.auth',
      template_folder=abspath(join(dirname(__file__), 'templates', 'auth')),
      url_prefix=self.config['URL_PREFIX']
    )

  def _create_login_manager(self):

    login_manager = LoginManager()
    login_manager.login_view = self.config['URL_PREFIX'] + '/sign_in'
    login_manager.login_message = 'Please sign in'

    @login_manager.user_loader
    def load_user(id):
      """Return the user from his email.

      Necessary for flask.login module.
        
      """
      return User.get_from_id(id)

    return login_manager

  def _before_register(self, project):
    """Will be called right before the blueprint is registered."""

    self.blueprint = self._create_blueprint(project)
    self.login_manager = self._create_login_manager()

  def _after_register(self, project):
    """Will be called right after the blueprint is registered."""

    self.login_manager.setup_app(project.app)

    if self.config['PROTECT_ALL_VIEWS']:

      @project.app.before_request
      def check_if_logged_in():
        if (request.blueprint != 'auth'
            and request.endpoint # favicon
            and not request.endpoint == 'static' # static files
            and not current_user.is_authenticated()):
          return self.login_manager.unauthorized()
        return None

class GoogleAuthManager(AuthManager):

  config = {
    'URL_PREFIX': '/auth',
    'PROTECT_ALL_VIEWS': True,
    'CLIENT_ID': '',
    'AUTHORIZED_EMAILS': None,
    'CALLBACK_URL': '/oauth2callback',
    'ENDPOINTS': {
        'get_token_or_code': "https://accounts.google.com/o/oauth2/auth",
        'validate_token': "https://www.googleapis.com/oauth2/v1/tokeninfo",
        'get_token_from_code': "https://accounts.google.com/o/oauth2/token",
        'get_user_info': "https://www.googleapis.com/oauth2/v2/userinfo",
    },
    'SCOPES': {
        'email': "https://www.googleapis.com/auth/userinfo.email",
        'profile': "https://www.googleapis.com/auth/userinfo.profile"
    },
    'RESPONSE_TYPE': "token",
    'GRANT_TYPE': "authorization_code",
    'ACCESS_TYPE': "offline",
  }

  def _before_register(self, project):

    super(GoogleAuthManager, self)._before_register(project)

    emails = self.config['AUTHORIZED_EMAILS']
    if isinstance(emails, list):
      for email in emails:
        User.__all__[email] = User(email)
    elif isinstance(emails, str):
      for email in emails.split(','):
        email = email.strip()
        if email:
          User.__all__[email] = User(email)

    bp = self.blueprint

    @bp.route('/sign_in')
    def sign_in():
      """Sign in view."""
      values = {'sign_in_url': self.login_url}
      return render_template('google_sign_in.html', **values)

    @bp.route('/sign_out')
    def sign_out():
      """Sign out."""
      if current_user.is_authenticated():
        current_user.info('Signed out.')
        logout_user()
        flash('Goodbye')
      return redirect(url_for('.sign_in'))


    @bp.route(self.config['CALLBACK_URL'])
    def oauth2callback():
      """Handles OAuth2 callback.

      Callbacks from the Google API first arrive here. However, since the token
      information is stored after the hash in the URL, Flask can't process it
      directly. Therefore this page renders a page with only JavaScript which
      then catches the token information and redirects to ``catch_token``.

      """
      values = {'catch_token_url': url_for('.catch_token')}
      return render_template('get_token_from_hash.html', **values)

    @bp.route('/catch_token')
    def catch_token():
      """Catches the token and signs in the user if it passes validation.

      If the user got to the sign in page from an non anonymous authorized page
      he will be directly redirected there after sign in. Otherwise he will be
      redirected to the home page.

      """
      token = request.args['access_token']
      if not self.validate_token(token):
        flash('Invalid token')
        return redirect(url_for('.sign_in'))
      user_infos = self.get_user_info_from_token(token)
      user = User.get_from_id(user_infos['email'])
      if user:
        login_user(user)
        user.info('Signed in.')
        return redirect(request.args['state'])
      else:
        current_app.logger.warn('%s tried to sign in.' % user_infos['email'])
        flash('Unauthorized')
        return redirect(url_for('.sign_in'))

  @property
  def login_url(self):
    """Combines the endpoint with the parameters to generate the API url."""
    return self.config['ENDPOINTS']['get_token_or_code'] + '?' + urlencode(self.get_params())
    
  def get_params(self):
    """Builds the dictionary of parameters required the API request."""
    return {
      'scope': self.config['SCOPES']['email'],
      'redirect_uri': request.url_root[:-1] + self.blueprint.url_prefix + self.config['CALLBACK_URL'],
      'response_type': self.config['RESPONSE_TYPE'],
      'state': request.args['next'] if 'next' in request.args else '/',
      'client_id': self.config['CLIENT_ID']
    }

  def validate_token(self, token):
    """Checks if the token is valid."""
    url = self.config['ENDPOINTS']['validate_token'] + '?access_token=' + token
    req = Request(url)
    token_info = loads(urlopen(req).read())
    return False if 'error' in token_info else True

  def get_user_info_from_token(self, token):
    """Grabs user email from token. """
    url = self.config['ENDPOINTS']['get_user_info']
    headers = {'Authorization': 'Bearer ' + token}
    req = Request(url, headers=headers)
    res = loads(urlopen(req).read())
    return res




#!/usr/bin/env python

"""Project module."""

from celery.signals import task_postrun
from ConfigParser import SafeConfigParser
from flask import abort
from os.path import abspath, dirname, join, sep, split, splitext
from re import match, sub
from sqlalchemy import create_engine  
from sqlalchemy.exc import InvalidRequestError
from sqlalchemy.orm import scoped_session, sessionmaker
from weakref import proxy
from werkzeug.local import LocalProxy

from .util import convert

class ProjectImportError(Exception):

  """Generic project import error.
  
  This will be raised for missing or invalid configuration files.
  
  """

  pass

class Project(object):

  """Project class.

  Global container for the Flask and Celery apps and SQLAlchemy database
  object.
  
  """

  __current__ = None

  config = {
    'PROJECT': {
      'NAME': '',
      'DOMAIN': '',
      'SUBDOMAIN': '',
      'MODULES': '',
      'APP_FOLDER': 'app',
      'APP_STATIC_FOLDER': 'static',
      'APP_TEMPLATE_FOLDER': 'templates',
    },
    'ENGINE': {
      'URL': 'sqlite://',
      'POOL_RECYCLE': 3600,
      'POOL_SIZE': 10,
      'CONVERT_UNICODE': True,
    },
    'APP': {
      'SECRET_KEY': 'a_default_unsafe_key',
    },
    'CELERY': {
      'BROKER_URL': 'redis://',
      'CELERY_RESULT_BACKEND': 'redis://',
      'CELERY_SEND_EVENTS': True
    },
  }

  def __init__(self, config_path):

    config = self._parse_config(config_path)
    for key in config:
      if key in self.config:
        self.config[key].update(config[key])
      else:
        self.config[key] = config[key]

    self.root_dir = dirname(abspath(config_path))
    self.domain = (
      self.config['PROJECT']['DOMAIN'] or
      sub(r'\W+', '_', self.config['PROJECT']['NAME'].lower())
    )
    self.subdomain = (
      self.config['PROJECT']['SUBDOMAIN'] or
      splitext(config_path)[0].replace(sep, '-')
    )

    assert Project.__current__ is None, 'More than one project.'
    Project.__current__ = proxy(self)

    self.app = None
    self.celery = None
    self.session = None
    self._engine = None
    self._managers = []
    self._before_startup = []

  def __repr__(self):
    return '<Project %r, %r>' % (self.config['PROJECT']['NAME'], self.root_dir)

  def register_manager(self, manager, config_section=None):
    """Register a manager."""
    self._managers.append((manager, config_section))

  def before_startup(self, func):
    """Decorator, hook to run a function right before project starts."""
    self._before_startup.append(func)

  def make(self, app=False, celery=False):
    """Create all project components."""
    # core
    for mod in  ['app', 'celery']:
      __import__('flasker.core.%s' % mod)
    # project modules
    project_modules = self.config['PROJECT']['MODULES'].split(',') or []
    for mod in project_modules:
      __import__(mod.strip())
    # database
    engine_ops = dict((k.lower(), v) for k,v in self.config['ENGINE'].items())
    self._engine = create_engine(engine_ops.pop('url'), **engine_ops)
    self.session = scoped_session(sessionmaker(bind=self._engine))
    if app:
      @self.app.teardown_request
      def teardown_request_handler(exception=None):
        self._dismantle_database_connections()
    if celery:
      @task_postrun.connect
      def task_postrun_handler(*args, **kwargs):
        self._dismantle_database_connections()
    # managers
    for manager, config_section in self._managers or []:
      if config_section:
        for k, v in self.config[config_section].items():
          manager.config[k] = v
      manager._before_register(self)
      self.app.register_blueprint(manager.blueprint)
      manager._after_register(self)
    # final hook
    for func in self._before_startup or []:
      func(self)

  def _dismantle_database_connections(self, **kwrds):
    """Remove database connection.

    Has to be called after app request/job terminates or connections
    will leak.

    """
    try:
      self.session.commit()
    except InvalidRequestError as e:
      self.session.rollback()
      self.session.expunge_all()
    finally:
      self.session.remove()

  def _parse_config(self, config_path):
    """Read the configuration file and return values as a dictionary.

    Raises ProjectImportError if no configuration file can be read at the
    file path entered.

    """
    parser = SafeConfigParser()
    parser.optionxform = str    # setting options to case-sensitive
    try:
      with open(config_path) as f:
        parser.readfp(f)
    except IOError as e:
      raise ProjectImportError(
        'Unable to parse configuration file at %s.' % config_path
      )
    conf = dict(
      (s, dict((k, convert(v)) for (k, v) in parser.items(s)))
      for s in parser.sections()
    )
    # some conf checking
    if not conf['PROJECT']['NAME']:
      raise ProjectImportError('Missing project name.')
    return conf

  @classmethod
  def get_current_project(cls):
    """Hook for ``current_project`` proxy."""
    return Project.__current__

current_project = LocalProxy(lambda: Project.get_current_project())




#!/usr/bin/env python

from setuptools import find_packages, setup

setup(
    name='flasker',
    version='0.1.26',
    description='Flask, SQLAlchemy, and Celery integration',
    long_description=open('README.rst').read(),
    author='Matthieu Monsch',
    author_email='monsch@mit.edu',
    url='https://github.com/mtth/flasker',
    license='MIT',
    packages=find_packages(),
    classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: MIT License',
      'Topic :: System :: Distributed Computing',
      'Programming Language :: Python',
    ],
    install_requires=[
      'celery',
      'flask',
      'flask-login',
      'sqlalchemy',
      'redis',
      'flower'
    ],
    package_data={'flasker': [
      'ext/templates/**/*',
      'data/**/**/*',
    ]},
    include_package_data=True,
    entry_points={'console_scripts': ['flasker = flasker.__main__:main']},
)
